package com.example.test1;

import java.util.Timer;
import java.util.TimerTask;

import com.ThermalExpert.ThermalExpert;
import com.ThermalExpert.ThermalExpert.ThermalExpertListener;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface.OnClickListener;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Rect;
import android.graphics.Bitmap.Config;
import android.graphics.drawable.BitmapDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.ViewGroup.LayoutParams;
import android.widget.ImageView;
import android.widget.ImageView.ScaleType;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;


public class MainActivity extends Activity {
	ThermalExpert 	te;
	ImageView		mTeImageView;
	
	Button 			playButton;
	Button 			calibrationButton;
	Button			colorButton;
	Button			flashButton;
	Button			agcButton;
	TextView		temperatureText;
	TextView		temperatureRectMax;
	TextView		temperatureRectAvg;
	TextView		temperatureRectMin;
	TextView		temperatureCoorText;
	TextView		temperatureRectCoor;
	
	Rect			mRectTemperature = new Rect();
	
	private TimerTask 	mTask;
	private Timer 		mTimer;	
	
	private Handler 	handler;
	private Runnable 	runnable, runnableDestroy;
	private AlertDialog dialog;
	
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
                
        mTeImageView = (ImageView) findViewById(R.id.image_preview);
        mTeImageView.setScaleType(ScaleType.CENTER);

        playButton = (Button) findViewById(R.id.play_button);
        playButton.setOnClickListener(mClickListener);
        
        flashButton = (Button) findViewById(R.id.flash_reload);
        flashButton.setOnClickListener(mClickListener);
        
        calibrationButton = (Button) findViewById(R.id.calibration_button);
        calibrationButton.setOnClickListener(mClickListener);
        
        colorButton = (Button) findViewById(R.id.color_button);
        colorButton.setOnClickListener(mClickListener);
        
        agcButton = (Button) findViewById(R.id.agc_onoff);
        agcButton.setOnClickListener(mClickListener);
        
        temperatureText = (TextView) findViewById(R.id.temperature_textview);
        temperatureCoorText = (TextView) findViewById(R.id.temperature_coor_textview);
        temperatureRectMax = (TextView) findViewById(R.id.temperature_rect_max);
        temperatureRectAvg = (TextView) findViewById(R.id.temperature_rect_avg);
        temperatureRectMin = (TextView) findViewById(R.id.temperature_rect_min);
        temperatureRectCoor = (TextView) findViewById(R.id.temperature_rect_coor);
        
        te = new ThermalExpert(thermalExpertListener, mTeImageView);
        te.Initial(getApplicationContext());  
        
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setMessage("Thermal Expert is not connected.\n This app is going to be closed.");
		dialog = builder.create();

	    LayoutParams params = dialog.getWindow().getAttributes();
	    params.width = 500;
	    params.height = 100;
	    dialog.getWindow().setAttributes((android.view.WindowManager.LayoutParams) params);
	    
		handler = new Handler();
		runnable = new Runnable() {
			@Override
			public void run() {
				dialog.show();
				Toast.makeText(getApplicationContext(), "Thermal Expert is not connected\n This app is going to be closed.", Toast.LENGTH_LONG).show();
			}
		};

		runnableDestroy = new Runnable() {
			@Override
			public void run() {
				dialog.dismiss();
			}
		};
		
		mTask = new TimerTask() {
			public void run() {
				if( !te.isUSBConnect() ) {
					handler.post(runnable);
					try {
						Thread.sleep(3000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					handler.post(runnableDestroy);

					moveTaskToBack(true);
					finish();
					android.os.Process.killProcess(android.os.Process.myPid());
				}
				te.FlashReload();
			}
		};

		mTimer = new Timer();
		mTimer.schedule(mTask, 2000);
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        if (id == R.id.action_settings) {
            return true;
        }
        return super.onOptionsItemSelected(item);
    }
    
	@Override
	protected void onResume() {
		super.onResume();
	}
	
	@Override
	protected void onPostResume() {
		super.onPostResume();
	}
	
	@Override
	protected void onRestart() {
		super.onRestart();
	};
	
	@Override
	protected void onDestroy() {
		super.onDestroy();
		
		mTimer.cancel();
		mTask.cancel();
    }	
	
	@Override
	protected void onPause() {
		super.onPause();
		
		te.StopReceive();
	}

	@Override
	protected void onUserLeaveHint() {
		super.onUserLeaveHint();
	}
    
	Button.OnClickListener mClickListener = new View.OnClickListener() {
		public void onClick(View v) {
			TextView playButtonText = (TextView)findViewById(R.id.play_button);
			TextView colorButtonText = (TextView)findViewById(R.id.color_button);
			switch (v.getId()) {
			case R.id.play_button:
				if( playButtonText.getText().toString().equals("play") ) {
					playButtonText.setText("stop");
					te.StopReceive();
				} else if( playButtonText.getText().toString().equals("stop") ) {
					playButtonText.setText("play");
					te.StartReceive();
				}
				break;
			case R.id.calibration_button :
				calibrationButton.setEnabled(false);
				te.CalibrationImage();				
				break;
				
			case R.id.color_button :
				if( colorButtonText.getText().toString().equals("Whitehot") ) {
					colorButtonText.setText("Blackhot");
					te.SetColorMap(1);
				} else if( colorButtonText.getText().toString().equals("Blackhot") ) {
					colorButtonText.setText("Iron");
					te.SetColorMap(2);
				} else if( colorButtonText.getText().toString().equals("Iron") ) {
					colorButtonText.setText("BlueRed");
					te.SetColorMap(3);
				} else if( colorButtonText.getText().toString().equals("BlueRed") ) {
					colorButtonText.setText("Medical");
					te.SetColorMap(4);
				} else if( colorButtonText.getText().toString().equals("Medical") ) {
					colorButtonText.setText("Purple");
					te.SetColorMap(5);
				} else if( colorButtonText.getText().toString().equals("Purple") ) {
					colorButtonText.setText("Purple Yellow");
					te.SetColorMap(6);
				} else if( colorButtonText.getText().toString().equals("Purple Yellow") ) {
					colorButtonText.setText("Dark Blue");
					te.SetColorMap(7);
				} else if( colorButtonText.getText().toString().equals("Dark Blue") ) {
					colorButtonText.setText("Cyan");
					te.SetColorMap(8);
				} else if( colorButtonText.getText().toString().equals("Cyan") ) {
					colorButtonText.setText("Rainbow");
					te.SetColorMap(9);
				} else if( colorButtonText.getText().toString().equals("Rainbow") ) {
					colorButtonText.setText("Whitehot");
					te.SetColorMap(0);
				}
				break;
			case R.id.flash_reload :
				flashButton.setEnabled(false);
				te.StopReceive();
				te.FlashReload();
				flashButton.setEnabled(true);
				break;
			case R.id.agc_onoff :
				if( agcButton.getText().toString().equals("Auto LevelSpan")) {
					double minTemp = te.GetAGCLowTemp();
					double maxTemp = te.GetAGCHighTemp();
					te.SetUserLevelSpan(true);
					te.SetLevelSpanMin(minTemp);
					te.SetLevelSpanMax(maxTemp);
					agcButton.setText("User LevelSpan");
				} else if( agcButton.getText().toString().equals("User LevelSpan")) {
					te.SetUserLevelSpan(false);
					agcButton.setText("Auto LevelSpan");
				}
				break;			
			}
		}
	};

	public boolean onTouchEvent(MotionEvent event) {
		int x = (int)event.getX();
		int y = (int)event.getY();
		
		if(event.getAction() == MotionEvent.ACTION_DOWN ) {
			mRectTemperature.set(x, y, x, y);
		} else if(event.getAction() == MotionEvent.ACTION_UP) {
			if( Math.abs(mRectTemperature.left - x) >= 10 && Math.abs(mRectTemperature.top - y) >= 10 ) {
				mRectTemperature.set(mRectTemperature.left, mRectTemperature.top, x, y);
				double[] temp = te.GetRectTemperature(mRectTemperature);
				temperatureRectAvg.setText(String.format("%.2f",temp[0]));
				temperatureRectMax.setText(String.format("%.2f",temp[1]));
				temperatureRectMin.setText(String.format("%.2f",temp[2]));
				temperatureRectCoor.setText(String.format("(%d, %d),\n(%d, %d)", te.GetTemperatureRect().left, te.GetTemperatureRect().top, te.GetTemperatureRect().right, te.GetTemperatureRect().bottom));
			} else {
				double temp = te.GetPointTemperature(x, y);
				temperatureText.setText(String.format("%.2f", temp));
				temperatureCoorText.setText(String.format("(%d, %d)", te.GetTemperaturePoint().x, te.GetTemperaturePoint().y));
			}
		}
		return super.onTouchEvent(event);
	}
	
	private boolean isRecog = false;
	private ThermalExpertListener thermalExpertListener = new ThermalExpertListener() {
		int counter = 0;
		
		public void onUsbConnected() {
//			SetUserColorTable();
		}

		@Override
		public void onUsbDisconnected() {
			Toast.makeText(getApplicationContext(), "USB disconnected", Toast.LENGTH_SHORT).show();
			if( te != null ) {
				moveTaskToBack(true);
				finish();
				android.os.Process.killProcess(android.os.Process.myPid());
			}
		}

		@Override
		public void onFlashReadFinished() {
			te.StartReceive();
		}

		@Override
		public void onAGCTemperatureChanged() {
			
		}
		
		@Override
		public void onOneFrameFinished() {
			mTeImageView.setBackground(new BitmapDrawable(te.GetImage()));
		}

		@Override
		public void onCalibrationFinished() {
			calibrationButton.setEnabled(true);
		}
		
		public void onRecogNumber() {
			isRecog = true;
		}

//		int []ColorTable = new int[256];
//		private void SetUserColorTable() {
//			int i;
//
//			// Pseudo color for medical application
//			int temp[][] = { { 255, 255, 250 }, { 255, 255, 247 }, { 255, 255, 242 }, { 255, 255, 235 }, { 255, 254, 229 }, { 255, 254, 223 }, { 255, 253, 218 },
//					{ 255, 253, 215 }, { 255, 252, 210 }, { 255, 251, 204 }, { 255, 250, 197 }, { 255, 250, 191 }, { 255, 250, 184 }, { 255, 250, 176 },
//					{ 255, 249, 169 }, { 255, 248, 162 }, { 255, 247, 156 }, { 255, 247, 149 }, { 255, 245, 142 }, { 255, 245, 134 }, { 255, 244, 126 },
//					{ 255, 244, 118 }, { 255, 242, 111 }, { 255, 242, 104 }, { 255, 241, 98 }, { 255, 241, 93 }, { 255, 239, 85 }, { 255, 238, 77 },
//					{ 255, 237, 71 }, { 255, 236, 67 }, { 255, 234, 62 }, { 255, 233, 56 }, { 255, 232, 49 }, { 255, 231, 45 }, { 255, 230, 42 }, { 255, 228, 36 },
//					{ 255, 227, 32 }, { 255, 226, 28 }, { 255, 225, 23 }, { 255, 224, 19 }, { 255, 222, 18 }, { 255, 221, 15 }, { 255, 218, 11 }, { 255, 216, 10 },
//					{ 255, 215, 9 }, { 255, 213, 8 }, { 255, 211, 6 }, { 255, 209, 5 }, { 255, 208, 3 }, { 255, 207, 1 }, { 255, 206, 1 }, { 255, 205, 0 },
//					{ 255, 202, 0 }, { 255, 201, 0 }, { 255, 200, 1 }, { 255, 197, 0 }, { 255, 196, 1 }, { 255, 194, 1 }, { 255, 191, 1 }, { 255, 189, 0 },
//					{ 255, 187, 1 }, { 255, 186, 0 }, { 255, 184, 0 }, { 255, 182, 0 }, { 255, 180, 1 }, { 255, 180, 1 }, { 255, 178, 0 }, { 255, 177, 0 },
//					{ 255, 173, 0 }, { 255, 171, 0 }, { 255, 169, 1 }, { 255, 167, 2 }, { 255, 165, 2 }, { 255, 163, 0 }, { 255, 161, 1 }, { 255, 159, 1 },
//					{ 255, 157, 0 }, { 255, 155, 0 }, { 255, 153, 0 }, { 255, 150, 0 }, { 255, 149, 1 }, { 255, 147, 0 }, { 255, 145, 0 }, { 255, 143, 0 },
//					{ 255, 141, 0 }, { 255, 139, 0 }, { 255, 137, 0 }, { 255, 135, 0 }, { 255, 134, 0 }, { 255, 133, 0 }, { 255, 131, 0 }, { 255, 129, 0 },
//					{ 255, 127, 0 }, { 254, 125, 0 }, { 254, 124, 0 }, { 253, 123, 0 }, { 253, 122, 0 }, { 252, 121, 0 }, { 252, 120, 0 }, { 251, 119, 0 },
//					{ 251, 118, 0 }, { 250, 117, 0 }, { 250, 116, 0 }, { 249, 115, 0 }, { 249, 114, 0 }, { 248, 112, 0 }, { 248, 110, 0 }, { 247, 108, 0 },
//					{ 247, 106, 0 }, { 246, 104, 1 }, { 246, 102, 2 }, { 245, 100, 3 }, { 245, 98, 4 }, { 244, 96, 5 }, { 244, 94, 6 }, { 243, 92, 7 },
//					{ 243, 90, 8 }, { 242, 88, 9 }, { 242, 86, 10 }, { 241, 84, 11 }, { 241, 82, 12 }, { 240, 80, 13 }, { 240, 78, 14 }, { 239, 76, 15 },
//					{ 239, 74, 16 }, { 238, 72, 17 }, { 238, 70, 18 }, { 237, 68, 19 }, { 237, 66, 23 }, { 236, 64, 27 }, { 236, 62, 31 }, { 235, 60, 36 },
//					{ 235, 58, 41 }, { 234, 56, 46 }, { 233, 54, 51 }, { 232, 52, 56 }, { 231, 50, 61 }, { 230, 48, 66 }, { 229, 46, 71 }, { 228, 44, 76 },
//					{ 227, 42, 81 }, { 226, 40, 86 }, { 225, 38, 91 }, { 224, 36, 96 }, { 223, 34, 99 }, { 222, 32, 102 }, { 221, 30, 105 }, { 220, 28, 108 },
//					{ 219, 26, 111 }, { 218, 25, 114 }, { 217, 24, 117 }, { 216, 23, 120 }, { 215, 22, 123 }, { 214, 21, 126 }, { 213, 20, 129 }, { 212, 19, 132 },
//					{ 210, 18, 135 }, { 209, 17, 138 }, { 208, 16, 140 }, { 208, 15, 142 }, { 207, 14, 144 }, { 206, 13, 146 }, { 205, 12, 147 }, { 204, 11, 147 },
//					{ 202, 10, 148 }, { 200, 9, 148 }, { 198, 8, 149 }, { 196, 7, 149 }, { 194, 6, 150 }, { 192, 5, 150 }, { 190, 4, 151 }, { 188, 3, 151 },
//					{ 186, 2, 152 }, { 184, 1, 152 }, { 182, 0, 153 }, { 180, 0, 153 }, { 178, 0, 154 }, { 176, 0, 154 }, { 174, 0, 155 }, { 172, 0, 155 },
//					{ 170, 0, 156 }, { 168, 0, 156 }, { 166, 0, 157 }, { 164, 0, 157 }, { 162, 0, 158 }, { 160, 0, 158 }, { 158, 0, 159 }, { 156, 0, 159 },
//					{ 154, 0, 160 }, { 152, 0, 160 }, { 150, 0, 161 }, { 148, 0, 161 }, { 146, 0, 162 }, { 144, 1, 162 }, { 141, 0, 163 }, { 138, 0, 163 },
//					{ 135, 0, 164 }, { 132, 0, 164 }, { 129, 0, 165 }, { 126, 0, 165 }, { 123, 0, 166 }, { 120, 0, 166 }, { 117, 0, 165 }, { 114, 0, 164 },
//					{ 111, 0, 163 }, { 108, 0, 162 }, { 105, 0, 161 }, { 102, 0, 160 }, { 99, 0, 159 }, { 96, 0, 158 }, { 93, 0, 157 }, { 90, 0, 156 },
//					{ 87, 0, 155 }, { 84, 0, 154 }, { 81, 0, 153 }, { 78, 0, 153 }, { 75, 0, 152 }, { 72, 0, 152 }, { 69, 0, 151 }, { 66, 0, 151 }, { 63, 0, 150 },
//					{ 60, 0, 150 }, { 57, 0, 148 }, { 54, 0, 148 }, { 51, 0, 146 }, { 48, 0, 144 }, { 45, 0, 142 }, { 42, 0, 140 }, { 39, 0, 138 }, { 36, 0, 136 },
//					{ 34, 0, 134 }, { 32, 0, 132 }, { 30, 0, 130 }, { 28, 0, 128 }, { 26, 0, 126 }, { 24, 0, 124 }, { 22, 0, 122 }, { 20, 0, 120 }, { 18, 0, 118 },
//					{ 16, 0, 116 }, { 14, 0, 111 }, { 12, 0, 106 }, { 10, 0, 101 }, { 8, 0, 96 }, { 6, 0, 91 }, { 4, 0, 86 }, { 2, 0, 81 }, { 0, 0, 76 },
//					{ 0, 0, 71 }, { 0, 0, 66 }, { 0, 0, 61 }, { 0, 0, 56 }, { 1, 0, 51 }, { 1, 1, 46 }, { 0, 0, 41 }, { 0, 0, 36 } };
//
//			for (i = 0; i < 256; i++)
//				ColorTable[i] = Color.argb(0xff, temp[256-i-1][0] & 0xff, temp[256-i-1][1] & 0xff, temp[256-i-1][2] & 0xff);
//		}
	};
}
